import base64
import json
import urllib2

from sdetools.connector_plugin import (BaseProjectConnectorPlugin,
                                       ConnectorParam,
                                       ConnectorParamType,
                                       CustomAttribute,
                                       ProjectConnectorException)


class JiraConnectorPlugin(BaseProjectConnectorPlugin):
    """
    A sample Project Connector plugin, that retrieves project information
    from a JIRA server.
    """

    id = "JiraConnector"
    name = "JIRA Connector"
    description = "Retrieves project information from a JIRA instance"

    def configuration_params(self):
        server = ConnectorParam(name="server",
                                help_text="Example: https://myserver.atlassian.net",
                                label="Server")

        username = ConnectorParam(name="username",
                                  label="User Name")

        password = ConnectorParam(name="password",
                                  label="Password",
                                  param_type=ConnectorParamType.MASKED_STRING)

        return [server, username, password]

    def lookup_params(self):
        jira_project_key = ConnectorParam(name="jira_project_key", label="JIRA Project Key")
        return [jira_project_key]

    def expected_custom_attributes(self):
        jira_project_id = CustomAttribute(name="jira_project_id")
        lead = CustomAttribute(name="lead")
        return [jira_project_id, lead]

    def read_only_attributes(self):
        return ['jira_project_id']

    def fetch_project_data(self, configuration, lookup_data):
        project_details = self.get_project_details(configuration, lookup_data)

        return {
            'name': project_details['name'],
            'description': project_details['description'],
            'custom_attributes': {
                'jira_project_id': project_details['id'],
                'lead': project_details['lead']['name']
            }
        }

    def get_project_details(self, configuration, lookup_data):
        jira_project_key = lookup_data.get('jira_project_key', '')
        server = configuration['server']
        username = configuration['username']
        password = configuration['password']

        req_url = "%s/rest/api/2/project/%s" % (server, jira_project_key)
        request = urllib2.Request(req_url)
        base64string = base64.encodestring('%s:%s' % (username, password))
        request.add_header("Authorization", "Basic %s" % base64string.replace('\n', ''))

        try:
            result = urllib2.urlopen(request).read()
        except urllib2.HTTPError, err:
            if err.code == 401:
                error_message = 'Cannot authenticate to JIRA server. Make sure credentials are correct.'
            else:
                error_message = 'Cannot connect to to JIRA server. Reason: %s' % err

            raise ProjectConnectorException(error_message)

        except urllib2.URLError:
            raise ProjectConnectorException('Cannot connect to JIRA server (Check the server URL)')

        return json.loads(result)
