import base64
import json
import urllib2

from sdetools.connector_plugin import (BaseProjectConnectorPlugin,
                                       ConnectorParam,
                                       ConnectorParamType,
                                       CustomAttribute,
                                       ProjectConnectorException)


class RallyConnectorPlugin(BaseProjectConnectorPlugin):
    """
    A sample Project Connector plugin, that retrieves project information from a Rally server.
    """

    id = "RallyConnector"
    name = "Rally Connector"
    description = "A sample Project Connector plugin that retrieves project information from a Rally server"

    def configuration_params(self):
        server = ConnectorParam(name="server",
                                help_text="Example: https://rally1.rallydev.com",
                                label="Server")

        username = ConnectorParam(name="username",
                                  label="User Name")

        password = ConnectorParam(name="password",
                                  label="Password",
                                  param_type=ConnectorParamType.MASKED_STRING)

        return [server, username, password]

    def lookup_params(self):
        rally_project_id = ConnectorParam(name="rally_project_id", label="Rally Project ID")
        return [rally_project_id]

    def expected_custom_attributes(self):
        workspace = CustomAttribute(name="workspace")
        return [workspace]

    def read_only_attributes(self):
        return []

    def fetch_project_data(self, configuration, lookup_data):
        try:
            project_details = self.get_project_details(configuration, lookup_data)
            return {
                'name': project_details['Project']['_refObjectName'],
                'description': project_details['Project']['Description'],
                'custom_attributes': {
                    'workspace': project_details['Project']['Workspace']['_refObjectName']
                }
            }
        except KeyError:
            raise ProjectConnectorException('Cannot find project in Rally.')

    def get_project_details(self, configuration, lookup_data):
        rally_project_id = lookup_data.get('rally_project_id', '')
        server = configuration['server']
        username = configuration['username']
        password = configuration['password']

        req_url = "%s/slm/webservice/v2.0/project/%s" % (server, rally_project_id)
        request = urllib2.Request(req_url)
        base64string = base64.encodestring('%s:%s' % (username, password))
        request.add_header("Authorization", "Basic %s" % base64string.replace('\n', ''))

        try:
            result = urllib2.urlopen(request).read()
        except urllib2.HTTPError, err:
            if err.code == 401:
                error_message = 'Cannot authenticate to Rally server. Make sure credentials are correct.'
            else:
                error_message = 'Cannot connect to to Rally server. Reason: %s' % err

            raise ProjectConnectorException(error_message)

        except urllib2.URLError:
            raise ProjectConnectorException('Cannot connect to Rally server. (Check the server URL)')

        return json.loads(result)
