import urllib

from urlparse import parse_qsl
from sdetools.sdelib.testlib.response_generator import ResponseGenerator, RESPONSE_HEADERS


class VeracodeResponseGenerator(ResponseGenerator):

    def __init__(self, config, test_dir=None):
        self.base_url = '%s://%s' % (config['analysis_method'], config['analysis_server'])
        self.api_url = '%s/%s' % (self.base_url, 'api')
        resource_templates = ['app_list.xml', 'build_list.xml', 'report.xml']
        rest_api_targets = {
            '/api/4.0/getapplist.do$': 'get_app_list',
            '/api/4.0/getbuildlist.do\?app_id=\d+$': 'get_build_list',
            '/api/3.0/detailedreport.do\?build_id=\d+&app_id=\d+$': 'generate_detailed_report'
        }
        super(VeracodeResponseGenerator, self).__init__(rest_api_targets, resource_templates, test_dir)


    @staticmethod
    def encode_response(response):
        if response:
            return response.toxml()
        else:
            return ''

    @staticmethod
    def decode_data(data):
        try:
            qs = urllib.unquote_plus(data).decode('utf-8')

            return dict(parse_qsl(qs))
        except:
            return data

    """
    Response functions
    """

    def get_app_list(self, target, flag, data, method):
        if not flag:
            return RESPONSE_HEADERS, self.get_raw_xml_from_file('app_list')
        else:
            self.raise_error('500')

    def get_build_list(self, target, flag, data, method):
        if not flag:
            app_id = int(target.split('app_id=')[1])
            if app_id == 39624: 
                return RESPONSE_HEADERS, self.get_raw_xml_from_file('build_list')
            else:
                self.raise_error('404')
        else:
            self.raise_error('500')

    def generate_detailed_report(self, target, flag, data, method):
        if not flag:
            app_id = target.split('app_id=')[1]
            build_id = target.split('build_id=')[1].split('&')[0]
            if app_id and build_id:
                return RESPONSE_HEADERS, self.get_raw_xml_from_file('report')
            else:
                self.raise_error('404')
        else:
            self.raise_error('500')
