import unittest
import os

from sdetools.analysis_integration.tests.base_integration_test import BaseIntegrationTest
from sdetools.modules.import_veracode.veracode_integrator import VeracodeIntegrator, VeracodeAPI
from veracode_response_generator import VeracodeResponseGenerator
from sdetools.sdelib.commons import get_directory_of_current_module
from sdetools.sdelib.testlib.mock_response import MOCK_ALM_RESPONSE, MOCK_SDE_RESPONSE

class TestVeracodeIntegration(BaseIntegrationTest, unittest.TestCase):


    @classmethod
    def setUpClass(cls):
        sde_cmd = "import_veracode"
        mapping_file = 'mapping.xml'
        report_file = 'report.xml'
        integrator = VeracodeIntegrator
        cls.generator_cls = VeracodeResponseGenerator
        cls.test_dir = get_directory_of_current_module(cls)
        cls.mock_sde_response = MOCK_SDE_RESPONSE
        cls.mock_alm_response = MOCK_ALM_RESPONSE
        super(TestVeracodeIntegration, cls).setUpClass(sde_cmd, mapping_file, report_file, integrator)

    def setUp(self):
        super(TestVeracodeIntegration, self).setUp()
        self.init_response_generator()
        self.mock_alm_response.initialize(self.response_generator)

    def init_integrator(self):
    	vc_api = VeracodeAPI(self.config)
        self.integrator = self.integrator_cls(vc_api, self.config)
        self.config.import_custom_options()
        self.integrator.config['mapping_file'] = os.path.join(self.test_file_dir, self.mapping_file)
        self.integrator.config['report_file'] = os.path.join(self.test_file_dir, self.report_file)
        self.num_reports = 1

    def init_response_generator(self, resp_generator_cls=None):
        resp_generator_cls = resp_generator_cls or self.generator_cls
        self.response_generator = resp_generator_cls(self.config, self.test_dir)
 
    def expected_number_of_findings(self):
        return 2

    def test_import_from_api_endpoint(self):
        self.integrator.config['integration_mode'] = 'api'
        self.integrator.config['analysis_server'] = 'analysiscenter.veracode.com'
        self.integrator.retrieve_report('WebGoat') 
