import re

from sdetools.sdelib.testlib.response_generator import ResponseGenerator


class TFSResponseGenerator(ResponseGenerator):
    PROJECT_ID = '10000'
    TFS_ISSUE_TYPES = ["Bug", "New Feature", "Task", "Improvement", "Sub-task"]
    TFS_PRIORITY_NAMES = ['Blocker', 'Critical', 'Major', 'Minor', 'Trivial']
    TFS_STATUS_NAMES = ['Active', 'New', 'Resolved', 'Closed', 'Removed']
    TFS_TRANSITION_NAMES = ['Start Progress', 'Resolve Issue', 'Close Issue', 'Reopen Issue']

    def __init__(self, config, test_dir=None):
        self.base_url = '%s://%s' % (config['alm_method'], config['alm_server'])
        self.api_url = self.base_url
        self.project_key = config['alm_project']
        resource_templates = ['project.json', 'workItem.json', 'projectCollection.json']
        rest_api_targets = {
            '/_apis/projectcollections': 'get_project_collection',
            '/DefaultCollection/_apis/projects': 'get_projects',
            '/DefaultCollection/%s/_apis/wit/workitems/\$Task\?api-version=1.0' % self.project_key: 'create_work_items',
            '/DefaultCollection/_apis/wit/workitems/[0-9]*\?api-version=1.0': 'update_work_items',
            '/DefaultCollection/%s/_apis/wit/wiql\?api-version=1.0' % self.project_key: 'get_work_item_wiql',
            '/DefaultCollection/_apis/wit/workitems\?ids=[0-9]*': 'get_work_item'
        }
        super(TFSResponseGenerator, self).__init__(rest_api_targets, resource_templates, test_dir)

    def init_with_resources(self):
        project_data = {
            'self': '%s/project/%s' % (self.api_url, self.project_key),
            'id': self.PROJECT_ID,
            'key': self.project_key,
        }
        self.generator_add_resource('project', self.PROJECT_ID, project_data)


    """
       Response functions
    """
    def get_project_collection(self, target, flag, data, method):
        if not flag:
            return [], self.get_json_from_file('projectCollection')
        else:
            self.raise_error('500')

    def get_projects(self, target, flag, data, method):
        if not flag:
            return [], self.get_json_from_file('project')

        else:
            self.raise_error('500')

    def create_work_items(self, target, flag, data, method):
        if not flag and data:
            reformated_data = {'fields': {}}
            work_item_title = data[0]['value']
            if work_item_title is not None:
                work_item_id = self.extract_task_number_from_title(work_item_title)
                for field in data:
                    reformated_data['fields'][field['path'].split('/fields/')[1]] = field['value']
                reformated_data['id'] = work_item_id
                reformated_data['rev'] = 1
                reformated_data['_links'] = {
                    'html': {
                        'href': self.api_url + "/DefaultCollection/_apis/wit/workItems/" + work_item_id
                    }
                }

                self.generator_add_resource('workItem', work_item_id, reformated_data)
                return [], self.generator_get_resource('workItem', work_item_id)
        else:
            self.raise_error('500')

    def update_work_items(self, target, flag, data, method):
        if not flag:
            workID = (target.split('workitems/')[1]).split('?')[0]
            old_fields = self.generator_get_resource('workItem', workID, True)['fields']
            old_fields[data[0]['path'].split('/fields/')[1]] = data[0]['value']
            self.generator_update_resource('workItem', workID, old_fields)
            return [], []
        else:
            self.raise_error('500')

    def get_work_item_wiql(self, target, flag, data, method):
        if not flag and data:
            # Checking for well formed query
            if re.match("Select \[Id\] From WorkItems Where \[System.IterationPath\]='.*' and \[System.AreaPath\]='.*' and \[Title\] Contains 'T[0-9]*:'", data['query']):
                workItems = self.generator_get_all_resource('workItem')
                workID = data['query'].split("Contains '")[1].split("'")[0]
                for item in workItems:
                    if re.search(workID, item['fields']['System.Title']) is not None:
                        return [], {'count': 1, 'workItems': [item]}
                return [], [] # The query doesn't find a matching task, return empty
            else:
                self.raise_error('500')
        else:
            self.raise_error('500')

    def get_work_item(self, target, flag, data, method):
        if not flag:
            workID = target.split('ids=')[1]
            return [], {'count': 1, 'value': [self.generator_get_resource('workItem', workID)]}
        else:
            self.raise_error('500')
            
