import re
import json

from xml.dom import minidom
from sdetools.sdelib.testlib.response_generator import ResponseGenerator


class VersionOneResponseGenerator(ResponseGenerator):
    PROJECT_ID = '10000'

    def __init__(self, config, test_dir=None):
        self.base_url = '%s://%s' % (config['alm_method'], config['alm_server'])
        self.api_url = self.base_url
        self.project_key = config['alm_project']
        resource_templates = ['task.xml', 'task.json', 'loc.json', 'scope.json', 'taskStatus.json',
            'workitemPriority.json', 'primaryWorkItem.json', 'query.json'
        ]
        rest_api_targets = {
            '/rest-1.v1/Data/[A-Za-z]*': 'call_rest',
            '/meta.v1/[A-Za-z]*': 'get_meta',
            '/loc-2.v1': 'get_loc',
            '/query.v1': 'get_query'
        }
        super(VersionOneResponseGenerator, self).__init__(rest_api_targets, resource_templates, test_dir)

    @staticmethod
    def encode_response(response):
        if not response:
            return ''

        if isinstance(response, dict) or isinstance(response, list):
            return json.dumps(response)
        else:
            return response.toxml()

    def init_with_resources(self):
        query_data = {
                "Status":
                    {
                        "_oid": "TaskStatus:123"
                    },
                "Number": "TK-01094",
                "_oid": "Task:1711",
                "ChangeDate": "2015-11-02T12:48:25.2030000",
                "Name": "T178: Ask for consent from user prior to collecting personal information"
        }
        self.generator_add_resource('query', 1711, query_data)

    """
       Response functions
    """
    def call_rest(self, target, flag, data, method):
        if flag:
            self.raise_error('500')

        item = re.split("/rest-1.v1/Data/", target)[1]  # Grab desired item
        if item == "Scope":
            return {"content-type": "json"}, self.get_json_from_file('scope')
        elif item == "Task":
            if method == "POST":
                xml_data = minidom.parseString(data).getElementsByTagName('Relation')[0].getElementsByTagName('Asset')[0].attributes['idref'].value
                update_data = {'Status': {'_oid': xml_data}}
                self.generator_update_resource('query', '1711', update_data)
            return {"content-type": "json"}, self.get_json_from_file('task')
        elif item == "TaskStatus":
            return {"content-type": "json"}, self.get_json_from_file('taskStatus')
        elif item == "WorkitemPriority":
            return {"content-type": "json"}, self.get_json_from_file('workitemPriority')
        elif item == "PrimaryWorkitem":
            return {"content-type": "json"}, self.get_json_from_file('primaryWorkItem')
        elif re.match("Task/1711", item) and method == "POST":
            task_id = re.split("/", item)[1]
            xml_data = minidom.parseString(data).getElementsByTagName('Relation')[0].getElementsByTagName('Asset')[0].attributes['idref'].value
            update_data = {'Status': {'_oid': xml_data}}
            self.generator_update_resource('query', task_id, update_data)
            return {'content-type': 'json'}, self.generator_get_resource('query', '1711')
        else:
            self.raise_error('400')

    def get_meta(self, target, flag, data, method):
        if flag:
            self.raise_error('500')

        return {"content-type": "xml"}, self.get_xml_from_file('task')

    def get_loc(self, target, flag, data, method):
        if flag:
            self.raise_error('500')

        return {'content-type': 'json'}, self.get_json_from_file("loc")

    def get_query(self, target, flag, data, method):
        if flag:
            self.raise_error('500')

        return {'content-type': 'json'}, [[self.generator_get_resource('query', '1711')]]
