import json
from urllib import urlencode
from urllib2 import urlopen

from sdetools.sde_plugins.connector_plugin import (BaseProjectConnectorPlugin,
                                                   CustomAttribute,
                                                   ConnectorParam,
                                                   ProjectConnectorException)


class ImdbConnectorPlugin(BaseProjectConnectorPlugin):
    """
    A sample Project Connector plugin, that retrieves movie information from an IMDB web API.
    """

    id = 'ImdbConnector'
    name = 'IMDB Connector'
    description = 'Retrieves movie information from IMDB'

    def configuration_params(self):
        return []

    def lookup_params(self):
        return [
            ConnectorParam(name="external_id", label="Movie name or IMDB ID")
        ]

    def expected_custom_attributes(self):
        return [
            CustomAttribute(name='year'),
            CustomAttribute(name='rating'),
            CustomAttribute(name='director'),
            CustomAttribute(name='imdbID')
        ]

    def read_only_attributes(self):
        return []

    def fetch_project_data(self, configuration, lookup_data):
        external_id = lookup_data.get('external_id', '')
        param_id = 'i' if external_id.startswith('tt') else 't'
        url = 'http://www.omdbapi.com/?' + urlencode([
            (param_id, external_id),
            ('plot', 'short'),
            ('r', 'json'),
        ])

        content = urlopen(url).read()
        movie = json.loads(content)

        if movie.get('Response', 'False') != 'True':
            raise ProjectConnectorException(movie.get('Error', 'Movie not found!'))

        return {
            'name': movie.get('Title'),
            'description': movie.get('Plot'),
            'custom_attributes': {
                'year': movie.get('Year'),
                'rating': movie.get('Rated'),
                'director': movie.get('Director'),
                'imdbID': movie.get('imdbID'),
            }
        }
